package org.mai.dep810.spark

import org.apache.calcite.avatica.ColumnMetaData
import org.apache.spark.sql.SparkSession
import org.apache.spark.sql.types._
import org.log4s._


object MinimalSparkApp {

  val appName: String = "MinimalSparkApp"

  private[this] val logger = getLogger

  def main(args: Array[String]) {

    org.apache.log4j.LogManager.getLogger("com").setLevel(org.apache.log4j.Level.OFF)
    org.apache.log4j.LogManager.getLogger("org").setLevel(org.apache.log4j.Level.OFF)
    org.apache.log4j.LogManager.getLogger("org.mai").setLevel(org.apache.log4j.Level.INFO)

    logger.info(s"""Starting $appName job""")

    val dataFile = "iris.data"
    val spark = SparkSession.builder.appName(appName).getOrCreate()
    val logData = spark.read.textFile(dataFile).cache()

    val species1 = "Iris-setosa"
    val species2 = "Iris-versicolor"
    val species3 = "Iris-virginica"
    val numSpecies1 = logData.filter(line => line.contains(species1)).count()
    val numSpecies2 = logData.filter(line => line.contains(species2)).count()
    val numSpecies3 = logData.filter(line => line.contains(species3)).count()
    logger.info(s"Lines with $species1: $numSpecies1")
    logger.info(s"Lines with $species2: $numSpecies2")
    logger.info(s"Lines with $species3: $numSpecies3")


    //making schema
    val irisFields = "petalLength,petalWidth,sepalLengh,sepalWidth"
      .split(",")
      .map(fieldName => StructField(fieldName, DoubleType, nullable = false))

    val irisSchema = StructType(irisFields++Seq(StructField("species", StringType, nullable = false)))
    val irisData = spark
      .read
      .option("sep", ",")
      .schema(irisSchema)
      .csv(dataFile)

    irisData.show(10)
    irisData.printSchema()
    spark.stop()

    logger.info(s"""$appName job stopped""")
  }

}