package org.mai.dep810.spark.stackoverflow

import org.apache.spark.sql.SparkSession
import org.apache.spark.sql.functions._
import org.log4s.getLogger


object CsvRead {

  val appName: String = "CsvRead"

  private[this] val logger = getLogger

  def main(args: Array[String]): Unit = {
    org.apache.log4j.LogManager.getLogger("com").setLevel(org.apache.log4j.Level.OFF)
    org.apache.log4j.LogManager.getLogger("org").setLevel(org.apache.log4j.Level.OFF)
    org.apache.log4j.LogManager.getLogger("org.mai").setLevel(org.apache.log4j.Level.INFO)
    logger.info(s"""Starting $appName job""")

    val spark = SparkSession.builder.appName(appName).getOrCreate()

    val postsDF = spark
      .read
      .option("header", "true")
      .option("sep", ",")
      .csv("stackoverflow/Posts.csv")

    postsDF.show(10)
    postsDF.printSchema()

    import spark.implicits._

    val posts = postsDF.select(
      col("id").cast("integer"),
      col("title"),
      col("score").cast("integer"),
      col("viewCount").cast("integer"),
      col("answerCount").cast("integer"),
      col("commentCount").cast("integer"),
      col("ownerUserId").cast("integer"),
      col("lastEditorUserId").cast("integer"),
      col("acceptedAnswerId").cast("integer"),
      col("creationDate"),
      col("lastEditDate"),
      col("lastActivityDate")
    ).as[Post]

    val users = spark.read.parquet("stackoverflow/users.parquet").as[User]

    val ids = posts.join(users, posts.col("ownerUserId") === users.col("id")).select(
      posts.col("id").alias("postId"),
      users.col("id").alias("userId")
    )

    ids.show(10)

    spark.stop()
    logger.info(s"""$appName job stopped""")
  }
}
