package org.mai.dep810.spark.stackoverflow

import org.apache.spark.sql.SparkSession
import org.apache.spark.sql.functions._
import org.apache.spark.sql.hive.HiveContext
import org.log4s._

object JsonConverter {

  val appName: String = "JsonConverter"

  private[this] val logger = getLogger

  def main(args: Array[String]): Unit = {
    org.apache.log4j.LogManager.getLogger("com").setLevel(org.apache.log4j.Level.OFF)
    org.apache.log4j.LogManager.getLogger("org").setLevel(org.apache.log4j.Level.OFF)
    org.apache.log4j.LogManager.getLogger("org.mai").setLevel(org.apache.log4j.Level.INFO)
    logger.info(s"""Starting $appName job""")

    val spark = SparkSession.builder.appName(appName).getOrCreate()

    val users = spark
      .read
      .json("stackoverflow/Users.json")
    users.show(10)
    users.printSchema()

    import spark.implicits._

    val userTyped = users.select(
      col("id").cast("integer"),
      col("displayName"),
      col("location"),
      col("reputation").cast("integer"),
      col("views").cast("integer"),
      col("upVotes").cast("integer"),
      col("downVotes").cast("integer"),
      col("accountId").cast("integer"),
      col("creationDate"),
      col("lastAccessDate")
    ).as[User]

    logger.info(s"${userTyped.head}")

    //saving users into parquet files
    userTyped
      .write
      .mode("overwrite")
      .parquet("stackoverflow/users.parquet")

    //saving ussers as table in parquet format
    userTyped
      .write
      .mode("overwrite")
      .option("path", "stackoverflow/usersTable")
      .saveAsTable("users")

    //querying table
    val displayName = spark
      .sql("select displayName from users where id = 16010")
      //.select("displayName")
      .head
      .getAs[String](0)

    val hiveContext = new HiveContext(spark.sparkContext)
    userTyped.write.mode("overwrite").format("orc").save("stackoverflow/users.orc")

    logger.info(s"User $displayName has id 16010")

    spark.stop()

    logger.info(s"""$appName job stopped""")
  }
}


abstract class Entity(id: Int)

case class User(
                 id: Int,
                 displayName: String,
                 location: String,
                 reputation: Int,
                 views: Int,
                 upVotes: Int,
                 downVotes: Int,
                 accountId: Int,
                 creationDate: String,
                 lastAccessDate: String) extends Entity(id)

case class Post(
                 id: Int,
                 title: String,
                 score: Int,
                 viewCount: Int,
                 answerCount: Int,
                 commentCount: Int,
                 ownerUserId: Int,
                 lastEditorUserId: Int,
                 acceptedAnswerId: Int,
                 creationDate: String,
                 lastEditDate: String,
                 lastActivityDate: String) extends Entity(id)

case class Comment(
                    id: Int,
                    postId: Int,
                    score: Int,
                    creationDate: String,
                    userId: Int) extends Entity(id)
